/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.checkstyle5;

import org.apache.commons.lang.StringUtils;
import org.sonar.api.batch.maven.DependsUponCustomRules;
import org.sonar.api.batch.maven.MavenPlugin;
import org.sonar.api.batch.maven.MavenPluginHandler;
import org.sonar.api.batch.maven.MavenUtils;
import org.sonar.api.profiles.RulesProfile;
import org.sonar.api.resources.Project;
import org.sonar.api.utils.SonarException;

import java.io.File;
import java.io.IOException;

public class CheckstyleMavenPluginHandler implements MavenPluginHandler, DependsUponCustomRules {
  private RulesProfile rulesProfile;
  private CheckstyleRulesRepository checkstyleRulesRepository;

  public CheckstyleMavenPluginHandler(RulesProfile rulesProfile, CheckstyleRulesRepository checkstyleRulesRepository) {
    this.rulesProfile = rulesProfile;
    this.checkstyleRulesRepository = checkstyleRulesRepository;
  }

  public String getGroupId() {
    return MavenUtils.GROUP_ID_APACHE_MAVEN;
  }

  public String getArtifactId() {
    return "maven-checkstyle-plugin";
  }

  public String getVersion() {
    return "2.3-atlassian-1";
  }

  public boolean isFixedVersion() {
    return true;
  }

  public String[] getGoals() {
    return new String[]{"checkstyle"};
  }

  public void configure(Project project, MavenPlugin plugin) {
    configureBasicParameters(plugin);
    configureExclusions(project, plugin);
    configureConfigurationLocation(project, plugin);
  }

  protected void configureConfigurationLocation(Project project, MavenPlugin plugin) {
    try {
      String existingConfig = plugin.getParameter("configLocation");
      if (!project.getReuseExistingRulesConfig() || 
          (project.getReuseExistingRulesConfig() && StringUtils.isBlank(existingConfig))) {
        plugin.setParameter("configLocation", saveConfigXml(project).getCanonicalPath());
      }
    } catch (IOException e) {
      throw new SonarException("fail to save the checkstyle XML configuration", e);
    }
  }

  protected void configureBasicParameters(MavenPlugin plugin) {
    plugin.removeParameter("outputDirectory");
    plugin.removeParameter("outputFile");
    plugin.setParameter("outputFileFormat", "xml");
    plugin.setParameter("consoleOutput", "false");
    plugin.setParameter("enableRSS", "false");
    plugin.setParameter("skip", "false");
    plugin.setParameter("failsOnError", "false");
  }

  private File saveConfigXml(Project project) throws IOException {
    String configuration = checkstyleRulesRepository.exportConfiguration(rulesProfile);
    return project.getFileSystem().writeToWorkingDirectory(configuration, "checkstyle.xml");
  }

  protected void configureExclusions(Project project, MavenPlugin plugin) {
    if (project.getExclusionPatterns() != null && project.getExclusionPatterns().length > 0) {
      String excludes = StringUtils.join(project.getExclusionPatterns(), ",");
      plugin.setParameter("excludes", excludes);
    }
  }

}
