/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.checkstyle5;

import org.sonar.api.profiles.RulesProfile;
import org.sonar.api.resources.Java;
import org.sonar.api.rules.*;
import org.sonar.plugins.checkstyle5.xml.Module;
import org.sonar.plugins.checkstyle5.xml.Property;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CheckstyleRulesRepository extends AbstractImportableRulesRepository<Java, CheckstyleRulePriorityMapper> implements ConfigurationExportable {

  public CheckstyleRulesRepository(Java java) {
    super(java, new CheckstyleRulePriorityMapper());
  }

  @Override
  public String getRepositoryResourcesBase() {
    return "org/sonar/plugins/checkstyle5";
  }

  @Override
  public Map<String, String> getBuiltInProfiles() {
    Map<String, String> defaults = new HashMap<String, String>();
    defaults.put(RulesProfile.SONAR_WAY_NAME, "profile-sonar-way.xml");
    defaults.put(RulesProfile.SONAR_WAY_FINDBUGS_NAME, "profile-sonar-way.xml");
    defaults.put(RulesProfile.SUN_CONVENTIONS_NAME, "profile-sun-conventions.xml");
    return defaults;
  }

  public String exportConfiguration(RulesProfile activeProfile) {
    Module module = toXStream(activeProfile.getActiveRulesByPlugin(CheckstylePlugin.KEY));
    module.getOrCreateChild("TreeWalker" + Module.MODULE_SEPARATOR + "FileContentsHolder");
    module.getOrCreateChild("SuppressionCommentFilter");
    return addXmlHeader(module.toXml());
  }

  public List<ActiveRule> importConfiguration(String xml, List<Rule> rules) {
    List<ActiveRule> activeRules = new ArrayList<ActiveRule>();
    Module module = Module.fromXml(xml);
    buildActiveRulesFromXStream(module, "", activeRules, rules);
    return activeRules;
  }

  protected Module toXStream(List<ActiveRule> activeRules) {
    Module root = new Module("");

    for (ActiveRule activeRule : activeRules) {
      if (activeRule.getRule().getPluginName().equals(CheckstylePlugin.KEY)) {
        String configKey = activeRule.getRule().getConfigKey();
        Module module = root.getOrCreateChild(configKey);
        List<Property> properties = new ArrayList<Property>();
        properties.add(new Property("severity", getRulePriorityMapper().to(activeRule.getPriority())));
        for (ActiveRuleParam activeRuleParam : activeRule.getActiveRuleParams()) {
          if (activeRuleParam.getValue() != null && activeRuleParam.getValue().length() != 0) {
            properties.add(new Property(activeRuleParam.getRuleParam().getKey(), activeRuleParam.getValue()));
          }
        }
        module.setProperties(properties);
      }
    }
    if (root.getChildren().isEmpty()) {
      return new Module("Checker", null);
    }
    return root.getChildren().get(0);
  }


  protected String addXmlHeader(String xml) {
    return new StringBuilder()
        .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
        .append("<!DOCTYPE module PUBLIC \"-//Puppy Crawl//DTD Check Configuration 1.2//EN\" \"http://www.puppycrawl.com/dtds/configuration_1_2.dtd\">")
        .append("<!-- generated by Sonar -->\n")
        .append(xml)
        .toString();
  }


  protected void buildActiveRulesFromXStream(Module module, String modulePath, List<ActiveRule> activeRules, List<Rule> rules) {
    if (module.getChildren() == null || module.getChildren().isEmpty()) {
      for (Rule rule : rules) {
        if (rule.getConfigKey().equals(modulePath)) {
          RulePriority rulePriority = getRulePriority(module);
          ActiveRule activeRule = new ActiveRule(null, rule, rulePriority);
          activeRule.setActiveRuleParams(getActiveRuleParams(module, rule, activeRule));
          activeRules.add(activeRule);
          return;

        }
      }
    } else {
      String baseModulePath = modulePath.length() == 0 ? module.getName() + Module.MODULE_SEPARATOR :
          modulePath + Module.MODULE_SEPARATOR;
      for (Module child : module.getChildren()) {
        buildActiveRulesFromXStream(child, baseModulePath + child.getName(), activeRules, rules);
      }
    }
  }

  private RulePriority getRulePriority(Module module) {
    if (module.getProperties() != null) {
      for (Property property : module.getProperties()) {
        if ("severity".equals(property.getName())) {
          return getRulePriorityMapper().from(property.getValue());
        }
      }
    }
    return null;
  }

  private List<ActiveRuleParam> getActiveRuleParams(Module module, Rule rule, ActiveRule activeRule) {
    List<ActiveRuleParam> activeRuleParams = new ArrayList<ActiveRuleParam>();
    if (module.getProperties() != null) {
      for (Property property : module.getProperties()) {
        if (rule.getParams() != null) {
          for (RuleParam ruleParam : rule.getParams()) {
            if (ruleParam.getKey().equals(property.getName())) {
              activeRuleParams.add(new ActiveRuleParam(activeRule, ruleParam, property.getValue()));
            }
          }
        }
      }
    }
    return activeRuleParams;
  }

}
