/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.checkstyle5.xml;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

@XStreamAlias("module")
public class Module implements Comparable<String> {

  public static final String MODULE_SEPARATOR = "/";

  @XStreamAsAttribute
  private String name;

  @XStreamImplicit
  private List<Module> children;

  @XStreamImplicit(itemFieldName = "property")
  private List<Property> properties;

  @XStreamImplicit(itemFieldName = "metadata")
  private List<Metadata> metadata;

  public Module(String name, Module parent, List<Property> properties) {
    this.name = name;
    this.properties = properties;
    this.children = new ArrayList<Module>();
    if (parent != null) {
      parent.addChild(this);
    }
  }

  public Module(String name, Module parent) {
    this(name, parent, null);
  }

  public Module(String name) {
    this(name, null, null);
  }

  public void addChild(Module child) {
    children.add(child);
  }

  public String getName() {
    return name;
  }

  public void setProperties(List<Property> properties) {
    this.properties = properties;
  }

  public List<Property> getProperties() {
    return properties;
  }

  public List<Module> getChildren() {
    return children;
  }

  public int compareTo(String o) {
    return o.compareTo(name);
  }

  public int getChildPosition(String nameToFind) {
    int i = 0;
    for (Module child : children) {
      if (child.getName().equals(nameToFind)) {
        return i;
      }
      i++;
    }
    return -1;
  }

  public Module getOrCreateChild(String configKey) {
    Module result = this;
    String[] modules = StringUtils.split(configKey, MODULE_SEPARATOR);
    for (int modulePos = 0; modulePos < modules.length; modulePos++) {
      String moduleName = modules[modulePos];
      if (result.getChildren().isEmpty()) {
        Module child = new Module(moduleName);
        result.addChild(child);
        result = child;
      } else {
        int index = result.getChildPosition(moduleName);
        // Create new module if it doesn't exists or if it's the last one (To allow same rule activation)
        if (index < 0 || modulePos == modules.length - 1) {
          Module child = new Module(moduleName);
          result.addChild(child);
          result = child;
        } else {
          result = result.getChildren().get(index);
        }
      }
    }
    return result;
  }

  private static XStream newXStream() {
    XStream xStream = new XStream();
    xStream.processAnnotations(Module.class);
    xStream.processAnnotations(Property.class);
    xStream.processAnnotations(Metadata.class);
    return xStream;
  }

  public static Module fromXml(String xml) {
    InputStream input = null;
    try {
      input = IOUtils.toInputStream(xml, "UTF-8");
      return (Module) newXStream().fromXML(input);

    } catch (IOException e) {
      throw new RuntimeException("can't read configuration file", e);

    } finally {
      IOUtils.closeQuietly(input);
    }
  }

  public String toXml() {
    return newXStream().toXML(this);
  }

}
