/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.checkstyle5;

import org.junit.Test;
import org.mockito.ArgumentMatcher;
import static org.mockito.Matchers.anyObject;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.argThat;
import static org.mockito.Mockito.*;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.sonar.api.batch.SensorContext;
import org.sonar.api.profiles.RulesProfile;
import org.sonar.api.resources.DefaultProjectFileSystem;
import org.sonar.api.resources.JavaFile;
import org.sonar.api.resources.Project;
import org.sonar.api.rules.*;
import org.sonar.api.test.IsViolation;

import javax.xml.stream.XMLStreamException;
import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;

public class CheckstyleViolationsXmlParserTest {

  private void parse(SensorContext context, String xmlPath) throws IOException, URISyntaxException, XMLStreamException {
    DefaultProjectFileSystem fileSystem = mock(DefaultProjectFileSystem.class);
    when(fileSystem.getSourceDirs()).thenReturn(Arrays.asList(new File("/test/src/main/java")));

    Project project = mock(Project.class);
    when(project.getFileSystem()).thenReturn(fileSystem);


    RulesManager manager = mock(RulesManager.class);
    when(manager.getPluginRule(anyString(), anyString())).thenAnswer(new Answer<Rule>() {
      public Rule answer(InvocationOnMock invocation) {
        Object[] args = invocation.getArguments();
        return new Rule((String) args[1], (String) args[1], null, (String) args[0], "");
      }
    });
    RulesProfile profile = mock(RulesProfile.class);
    when(profile.getActiveRule(anyString(), anyString())).thenReturn(new ActiveRule(null, null, RulePriority.MINOR));
    CheckstyleViolationsXmlParser parser = new CheckstyleViolationsXmlParser(project, context, manager, profile);

    File xmlFile = new File(getClass().getResource(xmlPath).toURI());
    parser.parse(xmlFile);
  }

  @Test
  public void shouldFindAllClassViolations() throws IOException, URISyntaxException, XMLStreamException {
    SensorContext context = mock(SensorContext.class);
    parse(context, "/org/sonar/plugins/checkstyle5/CheckstyleViolationsXmlParserTest/checkstyle-full-result.xml");
    verify(context, times(22)).saveViolation((Violation) anyObject());
  }

  @Test
  public void shouldNotParseHtmlFiles() throws IOException, URISyntaxException, XMLStreamException {
    SensorContext context = mock(SensorContext.class);
    parse(context, "/org/sonar/plugins/checkstyle5/CheckstyleViolationsXmlParserTest/checkstyle-full-result.xml");

    class IsNotHtmlPackage extends ArgumentMatcher<Violation> {
      @Override
      public boolean matches(Object violation) {
        return !((Violation) violation).getResource().getName().equals("package") &&
            !((Violation) violation).getResource().getName().equals("package.html");
      }
    }

    verify(context, times(22)).saveViolation(argThat(new IsNotHtmlPackage()));
  }

  @Test
  public void shouldParseViolation() throws IOException, URISyntaxException, XMLStreamException {
    SensorContext context = mock(SensorContext.class);
    parse(context, "/org/sonar/plugins/checkstyle5/CheckstyleViolationsXmlParserTest/checkstyle-simple-result.xml");

    Violation wanted = new Violation(new Rule("checkstyle5", "com.puppycrawl.tools.checkstyle.checks.javadoc.JavadocTypeCheck"), new JavaFile("org.sonar.mvn.BaseSonarMojo"));
    wanted.setMessage("Missing a Javadoc comment.");
    verify(context).saveViolation(argThat(new IsViolation(wanted)));
  }
}
